@@ -79,11 +79,16 @@ struct TELEPATHY_QT4_NO_EXPORT AccountManager::Private
     ContactFactoryConstPtr contactFactory;
 
     // Introspection
+    int reintrospectionRetries;
+    bool gotInitialAccounts;
     QHash<QString, AccountPtr> incompleteAccounts;
     QHash<QString, AccountPtr> accounts;
     QStringList supportedAccountProperties;
 };
 
+static const int maxReintrospectionRetries = 5;
+static const int reintrospectionRetryInterval = 3;
+
 AccountManager::Private::Private(AccountManager *parent,
         const AccountFactoryConstPtr &accFactory, const ConnectionFactoryConstPtr &connFactory,
         const ChannelFactoryConstPtr &chanFactory, const ContactFactoryConstPtr &contactFactory)
@@ -94,7 +99,9 @@ AccountManager::Private::Private(AccountManager *parent,
       accFactory(accFactory),
       connFactory(connFactory),
       chanFactory(chanFactory),
-      contactFactory(contactFactory)
+      contactFactory(contactFactory),
+      reintrospectionRetries(0),
+      gotInitialAccounts(false)
 {
     debug() << "Creating new AccountManager:" << parent->busName();
 
@@ -227,8 +234,7 @@ void AccountManager::Private::addAccountForPath(const QString &path)
  * \ingroup clientam
  * \headerfile TelepathyQt4/account-manager.h <TelepathyQt4/AccountManager>
  *
- * \brief The AccountManager class provides an object representing a Telepathy
- * account manager.
+ * \brief The AccountManager class represents a Telepathy account manager.
  *
  * The remote object accessor functions on this object (allAccounts(),
  * validAccounts(), and so on) don't make any D-Bus calls; instead, they return/use
@@ -969,17 +975,25 @@ Client::AccountManagerInterface *AccountManager::baseInterface() const
     return mPriv->baseInterface;
 }
 
+void AccountManager::introspectMain()
+{
+    mPriv->introspectMain(mPriv);
+}
+
 void AccountManager::gotMainProperties(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<QVariantMap> reply = *watcher;
     QVariantMap props;
 
     if (!reply.isError()) {
+        mPriv->gotInitialAccounts = true;
+
         debug() << "Got reply to Properties.GetAll(AccountManager)";
         props = reply.value();
 
         if (props.contains(QLatin1String("Interfaces"))) {
             setInterfaces(qdbus_cast<QStringList>(props[QLatin1String("Interfaces")]));
+            mPriv->readinessHelper->setInterfaces(interfaces());
         }
 
         if (props.contains(QLatin1String("SupportedAccountProperties"))) {
@@ -991,15 +1005,24 @@ void AccountManager::gotMainProperties(QDBusPendingCallWatcher *watcher)
         foreach (const QString &path, paths) {
             mPriv->addAccountForPath(path);
         }
+
+        mPriv->checkIntrospectionCompleted();
     } else {
-        warning().nospace() <<
-            "GetAll(AccountManager) failed: " <<
-            reply.error().name() << ": " << reply.error().message();
+        if (mPriv->reintrospectionRetries++ < maxReintrospectionRetries) {
+            int retryInterval = reintrospectionRetryInterval;
+            if (reply.error().type() == QDBusError::TimedOut) {
+                retryInterval = 0;
+            }
+            QTimer::singleShot(retryInterval, this, SLOT(introspectMain()));
+        } else {
+            warning() << "GetAll(AccountManager) failed with" <<
+                reply.error().name() << ":" << reply.error().message();
+            mPriv->readinessHelper->setIntrospectCompleted(FeatureCore,
+                    false, reply.error());
+        }
     }
 
     watcher->deleteLater();
-
-    mPriv->checkIntrospectionCompleted();
 }
 
 void AccountManager::onAccountReady(Tp::PendingOperation *op)
@@ -1032,6 +1055,10 @@ void AccountManager::onAccountReady(Tp::PendingOperation *op)
 void AccountManager::onAccountValidityChanged(const QDBusObjectPath &objectPath,
         bool valid)
 {
+    if (!mPriv->gotInitialAccounts) {
+        return;
+    }
+
     QString path = objectPath.path();
 
     if (!mPriv->incompleteAccounts.contains(path) &&
@@ -1043,6 +1070,10 @@ void AccountManager::onAccountValidityChanged(const QDBusObjectPath &objectPath,
 
 void AccountManager::onAccountRemoved(const QDBusObjectPath &objectPath)
 {
+    if (!mPriv->gotInitialAccounts) {
+        return;
+    }
+
     QString path = objectPath.path();
 
     /* the account is either in mPriv->incompleteAccounts or mPriv->accounts */
