@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009-2010 Nokia Corporation
+ * @copyright Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,7 @@
 
 #include "TelepathyQt4/channel-factory.h"
 #include "TelepathyQt4/debug-internal.h"
+#include "TelepathyQt4/request-temporary-handler-internal.h"
 
 #include <TelepathyQt4/Account>
 #include <TelepathyQt4/AccountManager>
@@ -398,6 +400,13 @@ void ClientHandlerAdaptor::HandleChannels(const QDBusObjectPath &accountPath,
     SharedPtr<InvocationData> invocation(new InvocationData());
     QList<PendingOperation *> readyOps;
 
+    RequestTemporaryHandler *tempHandler = dynamic_cast<RequestTemporaryHandler *>(mClient);
+    if (tempHandler) {
+        debug() << "  This is a temporary handler for the Request & Handle API,"
+            << "giving an early signal of the invocation";
+        tempHandler->setDBusHandlerInvoked();
+    }
+
     PendingReady *accReady = accFactory->proxy(QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
             accountPath.path(),
             connFactory,
@@ -481,13 +490,19 @@ void ClientHandlerAdaptor::onReadyOpFinished(Tp::PendingOperation *op)
         SharedPtr<InvocationData> invocation = mInvocations.takeFirst();
 
         if (!invocation->error.isEmpty()) {
+            RequestTemporaryHandler *tempHandler = dynamic_cast<RequestTemporaryHandler *>(mClient);
+            if (tempHandler) {
+                debug() << "  This is a temporary handler for the Request & Handle API, indicating failure";
+                tempHandler->setDBusHandlerErrored(invocation->error, invocation->message);
+            }
+
             // We guarantee that the proxies were ready - so we can't invoke the client if they
             // weren't made ready successfully. Fix the introspection code if this happens :)
             invocation->ctx->setFinishedWithError(invocation->error, invocation->message);
             continue;
         }
 
-        debug() << "Invoking application observeChannels with" << invocation->chans.size()
+        debug() << "Invoking application handleChannels with" << invocation->chans.size()
             << "channels on" << mClient;
 
         mClient->handleChannels(invocation->ctx, invocation->acc, invocation->conn,
@@ -874,12 +889,12 @@ bool ClientRegistrar::registerClient(const AbstractClientPtr &client,
     QString busName = QLatin1String("org.freedesktop.Telepathy.Client.");
     busName.append(clientName);
     if (unique) {
-        // o.f.T.Client.<unique_bus_name>_<pointer> should be enough to identify
+        // o.f.T.Client.clientName.<unique_bus_name>_<pointer> should be enough to identify
         // an unique identifier
-        busName.append(QString(QLatin1String(".%1.x%2"))
+        busName.append(QString(QLatin1String(".%1_%2"))
                 .arg(mPriv->bus.baseService()
                     .replace(QLatin1String(":"), QLatin1String("_"))
-                    .replace(QLatin1String("."), QLatin1String("._")))
+                    .replace(QLatin1String("."), QLatin1String("_")))
                 .arg((intptr_t) client.data(), 0, 16));
     }
 
