@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -85,12 +86,12 @@ protected:
     virtual void setFinished();
 
 private Q_SLOTS:
-    void gotProperties(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotProperties(QDBusPendingCallWatcher *watcher);
 
-    void changeState();
-    void onStateChanged(uint state, uint stateReason);
-    void onInitialOffsetDefined(qulonglong initialOffset);
-    void onTransferredBytesChanged(qulonglong count);
+    TELEPATHY_QT4_NO_EXPORT void changeState();
+    TELEPATHY_QT4_NO_EXPORT void onStateChanged(uint state, uint stateReason);
+    TELEPATHY_QT4_NO_EXPORT void onInitialOffsetDefined(qulonglong initialOffset);
+    TELEPATHY_QT4_NO_EXPORT void onTransferredBytesChanged(qulonglong count);
 
 private:
     struct Private;
