@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,19 +72,19 @@ public:
     UIntList invalidHandles() const;
 
 private Q_SLOTS:
-    void onRequestHandlesFinished(QDBusPendingCallWatcher *watcher);
-    void onHoldHandlesFinished(QDBusPendingCallWatcher *watcher);
-    void onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *watcher);
-    void onHoldHandlesFallbackFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onRequestHandlesFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onHoldHandlesFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onHoldHandlesFallbackFinished(QDBusPendingCallWatcher *watcher);
 
 private:
     friend class ConnectionLowlevel;
 
-    PendingHandles(const ConnectionPtr &connection, HandleType handleType,
+    TELEPATHY_QT4_NO_EXPORT PendingHandles(const ConnectionPtr &connection, HandleType handleType,
             const QStringList &names);
-    PendingHandles(const ConnectionPtr &connection, HandleType handleType,
+    TELEPATHY_QT4_NO_EXPORT PendingHandles(const ConnectionPtr &connection, HandleType handleType,
             const UIntList &handles, const UIntList &alreadyHeld, const UIntList &notYetHeld);
-    PendingHandles(const QString &errorName, const QString &errorMessage);
+    TELEPATHY_QT4_NO_EXPORT PendingHandles(const QString &errorName, const QString &errorMessage);
 
     struct Private;
     friend struct Private;
