@@ -84,6 +84,8 @@ class StreamTubeChannel;
 class TextChannel;
 class TubeChannel;
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 typedef SharedPtr<AbstractClient> AbstractClientPtr;
 typedef SharedPtr<Account> AccountPtr;
 typedef SharedPtr<AccountCapabilityFilter> AccountCapabilityFilterPtr;
@@ -148,6 +150,8 @@ public:
         : SharedPtr<MethodInvocationContext<T1, T2, T3, T4, T5, T6, T7, T8> >(o) { }
 };
 
+#endif /* DOXYGEN_SHOULD_SKIP_THIS */
+
 } // Tp
 
 #endif
