@@ -7,11 +7,14 @@
 #include <QString>
 #include <QVariantMap>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
 #include <TelepathyQt4/Account>
 #include <TelepathyQt4/AccountManager>
 #include <TelepathyQt4/AbstractClientHandler>
 #include <TelepathyQt4/AbstractClientObserver>
 #include <TelepathyQt4/Channel>
+#include <TelepathyQt4/ChannelClassSpec>
 #include <TelepathyQt4/ChannelDispatchOperation>
 #include <TelepathyQt4/ChannelRequest>
 #include <TelepathyQt4/ClientHandlerInterface>
@@ -19,6 +22,7 @@
 #include <TelepathyQt4/ClientObserverInterface>
 #include <TelepathyQt4/ClientRegistrar>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Debug>
 #include <TelepathyQt4/MethodInvocationContext>
 #include <TelepathyQt4/PendingAccount>
@@ -204,8 +208,8 @@ class MyClient : public QObject,
     Q_OBJECT
 
 public:
-    static AbstractClientPtr create(const ChannelClassList &channelFilter,
-            const QStringList &capabilities,
+    static AbstractClientPtr create(const ChannelClassSpecList &channelFilter,
+            const AbstractClientHandler::Capabilities &capabilities,
             bool bypassApproval = false,
             bool wantsRequestNotification = false)
     {
@@ -214,8 +218,8 @@ public:
                         bypassApproval, wantsRequestNotification)));
     }
 
-    MyClient(const ChannelClassList &channelFilter,
-             const QStringList &capabilities,
+    MyClient(const ChannelClassSpecList &channelFilter,
+             const AbstractClientHandler::Capabilities &capabilities,
              bool bypassApproval = false,
              bool wantsRequestNotification = false)
         : AbstractClientObserver(channelFilter),
@@ -235,7 +239,7 @@ public:
             const QList<ChannelPtr> &channels,
             const ChannelDispatchOperationPtr &dispatchOperation,
             const QList<ChannelRequestPtr> &requestsSatisfied,
-            const QVariantMap &observerInfo)
+            const AbstractClientObserver::ObserverInfo &observerInfo)
     {
         mObserveChannelsAccount = account;
         mObserveChannelsConnection = connection;
@@ -249,10 +253,9 @@ public:
     }
 
     void addDispatchOperation(const MethodInvocationContextPtr<> &context,
-            const QList<ChannelPtr> &channels,
             const ChannelDispatchOperationPtr &dispatchOperation)
     {
-        mAddDispatchOperationChannels = channels;
+        mAddDispatchOperationChannels = dispatchOperation->channels();
         mAddDispatchOperationDispatchOperation = dispatchOperation;
 
         context->setFinished();
@@ -270,7 +273,7 @@ public:
             const QList<ChannelPtr> &channels,
             const QList<ChannelRequestPtr> &requestsSatisfied,
             const QDateTime &userActionTime,
-            const QVariantMap &handlerInfo)
+            const AbstractClientHandler::HandlerInfo &handlerInfo)
     {
         mHandleChannelsAccount = account;
         mHandleChannelsConnection = connection;
@@ -310,7 +313,7 @@ public:
     QList<ChannelPtr> mObserveChannelsChannels;
     ChannelDispatchOperationPtr mObserveChannelsDispatchOperation;
     QList<ChannelRequestPtr> mObserveChannelsRequestsSatisfied;
-    QVariantMap mObserveChannelsObserverInfo;
+    AbstractClientObserver::ObserverInfo mObserveChannelsObserverInfo;
 
     QList<ChannelPtr> mAddDispatchOperationChannels;
     ChannelDispatchOperationPtr mAddDispatchOperationDispatchOperation;
@@ -321,7 +324,7 @@ public:
     QList<ChannelPtr> mHandleChannelsChannels;
     QList<ChannelRequestPtr> mHandleChannelsRequestsSatisfied;
     QDateTime mHandleChannelsUserActionTime;
-    QVariantMap mHandleChannelsHandlerInfo;
+    AbstractClientHandler::HandlerInfo mHandleChannelsHandlerInfo;
     ChannelRequestPtr mAddRequestRequest;
     ChannelRequestPtr mRemoveRequestRequest;
     QString mRemoveRequestErrorName;
@@ -388,7 +391,7 @@ private:
     QString mChannelRequestPath;
     ChannelDispatchOperationAdaptor *mCDO;
     QString mCDOPath;
-    QStringList mClientCapabilities;
+    AbstractClientHandler::Capabilities mClientCapabilities;
     AbstractClientPtr mClientObject1;
     QString mClientObject1BusName;
     QString mClientObject1Path;
@@ -456,18 +459,18 @@ void TestClient::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    mConn->requestConnect();
-
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
     QCOMPARE(static_cast<uint>(mConn->status()),
-             static_cast<uint>(Connection::StatusConnected));
+             static_cast<uint>(ConnectionStatusConnected));
 
     // create a Channel by magic, rather than doing D-Bus round-trips for it
 
@@ -545,18 +548,12 @@ void TestClient::testRegister()
     // invalid client
     QVERIFY(!mClientRegistrar->registerClient(AbstractClientPtr(), QLatin1String("foo")));
 
-    mClientCapabilities.append(
-        QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".MediaSignalling/ice-udp=true"));
-    mClientCapabilities.append(
-        QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".MediaSignalling/audio/speex=true"));
+    mClientCapabilities.setICEUDPNATTraversalToken();
+    mClientCapabilities.setToken(TP_QT4_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING +
+            QLatin1String("/audio/speex=true"));
 
-    ChannelClassList filters;
-    QMap<QString, QDBusVariant> filter;
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
-                  QDBusVariant(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT)));
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
-                  QDBusVariant((uint) Tp::HandleTypeContact));
-    filters.append(filter);
+    ChannelClassSpecList filters;
+    filters.append(ChannelClassSpec::textChat());
     mClientObject1 = MyClient::create(filters, mClientCapabilities, false, true);
     QVERIFY(mClientRegistrar->registerClient(mClientObject1, QLatin1String("foo")));
     QVERIFY(mClientRegistrar->registeredClients().contains(mClientObject1));
@@ -565,12 +562,7 @@ void TestClient::testRegister()
     QVERIFY(mClientRegistrar->registerClient(mClientObject1, QLatin1String("foo")));
 
     filters.clear();
-    filter.clear();
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
-                  QDBusVariant(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_STREAMED_MEDIA)));
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
-                  QDBusVariant((uint) Tp::HandleTypeContact));
-    filters.append(filter);
+    filters.append(ChannelClassSpec::streamedMediaCall());
     mClientObject2 = MyClient::create(filters, mClientCapabilities, true, true);
     QVERIFY(mClientRegistrar->registerClient(mClientObject2, QLatin1String("foo"), true));
     QVERIFY(mClientRegistrar->registeredClients().contains(mClientObject2));
@@ -597,16 +589,26 @@ void TestClient::testRegister()
 void TestClient::testCapabilities()
 {
     QDBusConnection bus = mClientRegistrar->dbusConnection();
+    QStringList normalizedClientCaps = mClientCapabilities.allTokens();
+    normalizedClientCaps.sort();
+
+    QStringList normalizedHandlerCaps;
 
     // object 1
     ClientHandlerInterface *handler1Iface = new ClientHandlerInterface(bus,
             mClientObject1BusName, mClientObject1Path, this);
-    QCOMPARE(handler1Iface->Capabilities(), mClientCapabilities);
+
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyCapabilities(), &normalizedHandlerCaps));
+    normalizedHandlerCaps.sort();
+    QCOMPARE(normalizedHandlerCaps, normalizedClientCaps);
 
     // object 2
     ClientHandlerInterface *handler2Iface = new ClientHandlerInterface(bus,
             mClientObject2BusName, mClientObject2Path, this);
-    QCOMPARE(handler2Iface->Capabilities(), mClientCapabilities);
+
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyCapabilities(), &normalizedHandlerCaps));
+    normalizedHandlerCaps.sort();
+    QCOMPARE(normalizedHandlerCaps, normalizedClientCaps);
 }
 
 void TestClient::testRequests()
@@ -659,12 +661,23 @@ void TestClient::testObserveChannelsCommon(const AbstractClientPtr &clientObject
     ChannelDetailsList channelDetailsList;
     ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
     channelDetailsList.append(channelDetails);
+
+    QVariantMap observerInfo;
+    ObjectImmutablePropertiesMap reqPropsMap;
+    QVariantMap channelReqImmutableProps;
+    channelReqImmutableProps.insert(
+            QLatin1String(
+                TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Interface.DomainSpecific.IntegerProp"), 3);
+    channelReqImmutableProps.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Account"),
+            qVariantFromValue(QDBusObjectPath(mAccount->objectPath())));
+    reqPropsMap.insert(QDBusObjectPath(mChannelRequestPath), channelReqImmutableProps);
+    observerInfo.insert(QLatin1String("request-properties"), qVariantFromValue(reqPropsMap));
     observeIface->ObserveChannels(QDBusObjectPath(mAccount->objectPath()),
             QDBusObjectPath(mConn->objectPath()),
             channelDetailsList,
             QDBusObjectPath("/"),
             ObjectPathList() << QDBusObjectPath(mChannelRequestPath),
-            QVariantMap());
+            observerInfo);
     QCOMPARE(mLoop->exec(), 0);
 
     QCOMPARE(client->mObserveChannelsAccount->objectPath(), mAccount->objectPath());
@@ -672,6 +685,12 @@ void TestClient::testObserveChannelsCommon(const AbstractClientPtr &clientObject
     QCOMPARE(client->mObserveChannelsChannels.first()->objectPath(), mText1ChanPath);
     QVERIFY(client->mObserveChannelsDispatchOperation.isNull());
     QCOMPARE(client->mObserveChannelsRequestsSatisfied.first()->objectPath(), mChannelRequestPath);
+    QCOMPARE(client->mObserveChannelsRequestsSatisfied.first()->immutableProperties().contains(
+            QLatin1String(
+                TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Interface.DomainSpecific.IntegerProp")), true);
+    QCOMPARE(qdbus_cast<int>(client->mObserveChannelsRequestsSatisfied.first()->immutableProperties().value(
+            QLatin1String(
+                TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Interface.DomainSpecific.IntegerProp"))), 3);
 }
 
 void TestClient::testObserveChannels()
@@ -741,7 +760,8 @@ void TestClient::testHandleChannels()
     QCOMPARE(client1->mHandleChannelsRequestsSatisfied.first()->objectPath(), mChannelRequestPath);
     QCOMPARE(client1->mHandleChannelsUserActionTime.toTime_t(), mUserActionTime);
 
-    Tp::ObjectPathList handledChannels = handler1Iface->HandledChannels();
+    Tp::ObjectPathList handledChannels;
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
 
     // object 2
@@ -768,15 +788,16 @@ void TestClient::testHandleChannels()
     QCOMPARE(client2->mHandleChannelsRequestsSatisfied.first()->objectPath(), mChannelRequestPath);
     QCOMPARE(client2->mHandleChannelsUserActionTime.toTime_t(), mUserActionTime);
 
-    handledChannels = handler1Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
-    handledChannels = handler2Iface->HandledChannels();
+
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
 
     mClientRegistrar->unregisterClient(mClientObject1);
-    handledChannels = handler2Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
 
     g_object_unref(mText2ChanService);
@@ -784,7 +805,7 @@ void TestClient::testHandleChannels()
             SIGNAL(channelClosed()),
             SLOT(expectSignalEmission()));
     QCOMPARE(mLoop->exec(), 0);
-    handledChannels = handler2Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.isEmpty());
 }
 
