@@ -5,8 +5,13 @@
 
 #include <QtTest/QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingChannel>
 #include <TelepathyQt4/PendingContacts>
@@ -15,7 +20,7 @@
 
 #include <telepathy-glib/debug.h>
 
-#include <tests/lib/glib/contactlist/conn.h>
+#include <tests/lib/glib/contactlist2/conn.h>
 #include <tests/lib/test.h>
 
 using namespace Tp;
@@ -32,6 +37,7 @@ public:
 protected Q_SLOTS:
     void expectConnInvalidated();
     void expectPendingContactsFinished(Tp::PendingOperation *);
+    void expectPresencePublicationRequested(const Tp::Contacts &);
     void expectPresenceStateChanged(Tp::Contact::PresenceState);
     void expectAllKnownContactsChanged(const Tp::Contacts &added, const Tp::Contacts &removed,
             const Tp::Channel::GroupMemberChangeDetails &details);
@@ -51,6 +57,8 @@ private:
     ConnectionPtr mConn;
     QList<ContactPtr> mContacts;
     int mHowManyKnownContacts;
+    bool mGotPresenceStateChanged;
+    bool mGotPPR;
 };
 
 void TestConnRoster::expectConnInvalidated()
@@ -92,8 +100,11 @@ void TestConnRoster::expectAllKnownContactsChanged(const Tp::Contacts& added, co
     qDebug() << added.size() << " contacts added, " << removed.size() << " contacts removed";
     mHowManyKnownContacts += added.size();
     mHowManyKnownContacts -= removed.size();
-    QVERIFY(details.hasMessage());
-    QCOMPARE(details.message(), QLatin1String("add me now"));
+
+    if (details.hasMessage()) {
+        QCOMPARE(details.message(), QLatin1String("add me now"));
+    }
+
     if (mConn->contactManager()->allKnownContacts().size() != mHowManyKnownContacts) {
         qWarning() << "Contacts number mismatch! Watched value: " << mHowManyKnownContacts
                    << "allKnownContacts(): " << mConn->contactManager()->allKnownContacts().size();
@@ -103,9 +114,19 @@ void TestConnRoster::expectAllKnownContactsChanged(const Tp::Contacts& added, co
     }
 }
 
+void TestConnRoster::expectPresencePublicationRequested(const Tp::Contacts &contacts)
+{
+    Q_FOREACH(Tp::ContactPtr contact, contacts) {
+        QCOMPARE(static_cast<uint>(contact->publishState()),
+                 static_cast<uint>(Contact::PresenceStateAsk));
+    }
+
+    mGotPPR = true;
+}
+
 void TestConnRoster::expectPresenceStateChanged(Contact::PresenceState state)
 {
-    mLoop->exit(0);
+    mGotPresenceStateChanged = true;
 }
 
 void TestConnRoster::initTestCase()
@@ -146,14 +167,16 @@ void TestConnRoster::init()
 {
     initImpl();
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create(Contact::FeatureAlias));
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 }
 
 void TestConnRoster::testRoster()
@@ -180,6 +203,7 @@ void TestConnRoster::testRoster()
     QList<ContactPtr> pendingPublish;
     Q_FOREACH (const ContactPtr &contact,
             mConn->contactManager()->allKnownContacts()) {
+        QVERIFY(contact->requestedFeatures().contains(Contact::FeatureAlias));
         qDebug() << " contact:" << contact->id() <<
             "- subscription:" << contact->subscriptionState() <<
             "- publish:" << contact->publishState();
@@ -206,12 +230,19 @@ void TestConnRoster::testRoster()
 
     int i = 0;
 
+    QVERIFY(connect(mConn->contactManager().data(),
+                    SIGNAL(presencePublicationRequested(Tp::Contacts,QString)),
+                    SLOT(expectPresencePublicationRequested(Tp::Contacts))));
+
     Q_FOREACH (const ContactPtr &contact, mContacts) {
+        mGotPresenceStateChanged = false;
+        mGotPPR = false;
+
         QVERIFY(connect(contact.data(),
-                        SIGNAL(subscriptionStateChanged(Tp::Contact::PresenceState)),
+                        SIGNAL(subscriptionStateChanged(Tp::Contact::PresenceState,Tp::Channel::GroupMemberChangeDetails)),
                         SLOT(expectPresenceStateChanged(Tp::Contact::PresenceState))));
         QVERIFY(connect(contact.data(),
-                        SIGNAL(publishStateChanged(Tp::Contact::PresenceState)),
+                        SIGNAL(publishStateChanged(Tp::Contact::PresenceState,Tp::Channel::GroupMemberChangeDetails)),
                         SLOT(expectPresenceStateChanged(Tp::Contact::PresenceState))));
         if ((i % 2) == 0) {
             contact->requestPresenceSubscription(QLatin1String("please add me"));
@@ -219,7 +250,9 @@ void TestConnRoster::testRoster()
             contact->requestPresenceSubscription(QLatin1String("add me now"));
         }
 
-        QCOMPARE(mLoop->exec(), 0);
+        while (!mGotPresenceStateChanged && !mGotPPR) {
+            mLoop->processEvents();
+        }
 
         if ((i % 2) == 0) {
             // I asked to see his presence - he might have already accepted it, though
@@ -229,11 +262,13 @@ void TestConnRoster::testRoster()
             // if he accepted it already, one iteration won't be enough as the
             // first iteration will just flush the subscription -> Yes event
             while (contact->publishState() != Contact::PresenceStateAsk) {
-                QCOMPARE(mLoop->exec(), 0);
+                mLoop->processEvents();
             }
 
             contact->authorizePresencePublication();
-            QCOMPARE(mLoop->exec(), 0);
+            while (contact->publishState() != Contact::PresenceStateYes) {
+                mLoop->processEvents();
+            }
             // I authorized him to see my presence
             QCOMPARE(static_cast<uint>(contact->publishState()),
                      static_cast<uint>(Contact::PresenceStateYes));
@@ -242,19 +277,18 @@ void TestConnRoster::testRoster()
                      static_cast<uint>(Contact::PresenceStateYes));
 
             contact->removePresenceSubscription();
-            QCOMPARE(mLoop->exec(), 0);
-            QCOMPARE(static_cast<uint>(contact->subscriptionState()),
-                     static_cast<uint>(Contact::PresenceStateNo));
+
+            while (contact->subscriptionState() != Contact::PresenceStateNo) {
+                mLoop->processEvents();
+            }
         } else {
-            // I asked to see his presence - she might have already rejected it, though
+            // I asked to see her presence - she might have already rejected it, though
             QVERIFY(contact->subscriptionState() == Contact::PresenceStateAsk
                     || contact->subscriptionState() == Contact::PresenceStateNo);
 
             // If she didn't already reject it, wait until she does
-            if (contact->subscriptionState() != Contact::PresenceStateNo) {
-                QCOMPARE(mLoop->exec(), 0);
-                QCOMPARE(static_cast<uint>(contact->subscriptionState()),
-                        static_cast<uint>(Contact::PresenceStateNo));
+            while (contact->subscriptionState() != Contact::PresenceStateNo) {
+                mLoop->processEvents();
             }
         }
 
@@ -276,11 +310,13 @@ void TestConnRoster::testRoster()
     i = 0;
     Contact::PresenceState expectedPresenceState;
     Q_FOREACH (const ContactPtr &contact, pendingPublish) {
+        mGotPresenceStateChanged = false;
+
         QVERIFY(connect(contact.data(),
-                        SIGNAL(publishStateChanged(Tp::Contact::PresenceState)),
+                        SIGNAL(publishStateChanged(Tp::Contact::PresenceState,Tp::Channel::GroupMemberChangeDetails)),
                         SLOT(expectPresenceStateChanged(Tp::Contact::PresenceState))));
 
-        if ((i % 2) == 0) {
+        if ((i++ % 2) == 0) {
             expectedPresenceState = Contact::PresenceStateYes;
             contact->authorizePresencePublication();
         } else {
@@ -288,11 +324,12 @@ void TestConnRoster::testRoster()
             contact->removePresencePublication();
         }
 
-        QCOMPARE(mLoop->exec(), 0);
+        while (!mGotPresenceStateChanged) {
+            mLoop->processEvents();
+        }
+
         QCOMPARE(static_cast<uint>(contact->publishState()),
                  static_cast<uint>(expectedPresenceState));
-
-        ++i;
     }
 
     // Test allKnownContactsChanged.
@@ -300,7 +337,7 @@ void TestConnRoster::testRoster()
     // Cache the current value
     mHowManyKnownContacts = mConn->contactManager()->allKnownContacts().size();
     // Watch for contacts changed
-    QVERIFY(connect(mConn->contactManager(),
+    QVERIFY(connect(mConn->contactManager().data(),
                     SIGNAL(allKnownContactsChanged(Tp::Contacts,Tp::Contacts,
                             Tp::Channel::GroupMemberChangeDetails)),
                     SLOT(expectAllKnownContactsChanged(Tp::Contacts,Tp::Contacts,
@@ -326,7 +363,7 @@ void TestConnRoster::cleanup()
 {
     if (mConn) {
         // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
